from typing import Optional


async def selectOptionQuestion(
    question: str, min: int, max: int, nullPossible: Optional[bool] = False
):
    invalid_input = True
    while invalid_input:
        user_input = input(f"{question} ")
        if user_input == "" and nullPossible:
            invalid_input = False
        elif inputIsInt(user_input):
            if min <= int(user_input) <= max:
                invalid_input = False
        else:
            print(f"\x1B[31mInvalid input! Please input within\x1B[37m {min}-{max}")
    return user_input


def inputIsInt(input: str):
    try:
        val = int(input)
        return True
    except ValueError:
        try:
            val = float(input)
            return False
        except ValueError:
            return False



from alive_progress import alive_bar
import pandas as pd
from typing import Callable, Optional, TypeVar, Any, Dict


ArgsType = TypeVar("ArgsType", bound=Dict)


async def processCSVInChunks(
    csv_file: str,
    process_function: Callable[[pd.DataFrame, Any], pd.DataFrame],
    pf_args: ArgsType,
    chunk_size: Optional[int] = 10000,
):
    """
    Process any CSV file in chunks instead of whole file at once\n\n
    `csv_file`: Path to the csv file.\n
    `chunk_size`: Size of chunks to work with\n
    :strong:`process_function`: The function containing the main processing you want to get done.\n
    `pf_args`: Arguments for the function in a dict eg:-\n
                def deleteRowsInCSV(dataframe,delete_rows):\n
                    # ...delete rows from csv.\n
                await processCSVInChunks(path_to_csv,deleteRowsInCSV,{delete_rows:20})\n

    """
    # Create a generator to read the CSV file in chunks
    chunk_reader = pd.read_csv(csv_file, chunksize=chunk_size)

    # Process each chunk and concatenate the results
    processed_chunks = []
    total_chunks = int(sum(1 for row in open(csv_file, "r")) / chunk_size)
    with alive_bar(total_chunks, force_tty=True, bar="filling", spinner="waves") as bar:
        bar.title = "Processing file -> " + csv_file
        for chunk in chunk_reader:
            processed_chunk = process_function(chunk, pf_args)
            processed_chunks.append(processed_chunk)
            bar()
    # Concatenate the processed chunks into a single DataFrame
    df = pd.concat(processed_chunks, ignore_index=True)

    return df



from alive_progress import alive_bar
from typing import Iterable, Any


def progressBarIterator(
    iterable: Iterable[Any], description: str = "Progress", **kwargs
):
    with alive_bar(
        len(iterable), force_tty=True, bar="filling", spinner="waves", **kwargs
    ) as bar:
        bar.title = description
        for item in iterable:
            yield item
            bar()



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="pypeepa",
    version="1.3",
    author="Ishfaq Ahmed",
    author_email="ishfaqahmed0837@gmail.com",
    description="Custom built utilities for general use",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/IshfaqAhmedProg/PyPeepa",
    packages=find_packages(),
    install_requires=[
        "pandas>=1.0.0",
        "alive-progress>=1.6.0",
        "pyfiglet>=0.8.post1",
        "psutil>=5.9.5",
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
)



import concurrent.futures
import time
import psutil
from typing import Optional, Callable, Any, Iterable
from pypeepa.utils.loggingHandler import loggingHandler  # check
import logging


def concurrentFutures(
    process_function: Callable[[Any], Any],
    divided_tasks: Iterable[Any],
    thread_executor: Optional[bool] = False,  # Default to "process" executor
    max_workers: Optional[int] = 4,
    logger: Optional[logging.Logger] = None,
):
    """
    Wrapper function to run multiple instances of the same function parallely,\n


    @param: `process_function`: The function that contains your processing logic.\n
    @param: `divided_tasks`:  The parameters for the process_function, It contains the divided tasks parameters in an Iterable.\n
    @param: `thread_executor`: (Optional) If True then ThreadPoolExcecutor will be used.\n
    @param: `max_workers`: (Optional) The maximum number of parallel processes/threads.\n
    @param: `logger`: (Optional) You can pass a logging object if you already initialized one.\n


    @return: Returns the results in an array if there are any return values to the process_function
    """
    start = time.time()

    executor_class = (
        concurrent.futures.ProcessPoolExecutor
        if thread_executor == False
        else concurrent.futures.ThreadPoolExecutor
    )

    results = []  # To store results from process_function

    with executor_class(max_workers=max_workers) as executor:
        futures = []
        for task in divided_tasks:
            future = executor.submit(process_function, task)
            futures.append((task, future))

        for task, future in futures:
            try:
                # Start measuring time
                start_time = time.time()

                # Get the result from the future
                result = future.result()
                results.append(result)
                # Print memory usage
                process = psutil.Process()
                memory_usage = process.memory_info().rss / (1024 * 1024)  # in megabytes
                # Calculate time_taken
                time_taken = time.time() - start_time

                loggingHandler(
                    logger=logger,
                    log_mssg="Start: {}, Time taken: {}s, Memory usage:, {}, MB".format(
                        task, time_taken, memory_usage
                    ),
                )
            except Exception as e:
                loggingHandler(
                    logger=logger,
                    log_mssg="Error occurred on task: {} --> Error Report: {}".format(
                        task, e
                    ),
                )

    loggingHandler(
        logger=logger, log_mssg="Total time taken: {}s".format(time.time() - start)
    )

    return results



from .fileInteraction import (
    getFilePath,
    asyncListDir,
    asyncReadJSON,
    createDirectory,
    readData,
    writeFile,
)
from .userInteraction import (
    askYNQuestion,
    getInputFilePath,
    printArray,
    selectOptionQuestion,
    signature,
    progressBarIterator,
)
from .optimisations import processCSVInChunks, concurrentFutures, ProgressSaver
from .debugging import initLogging
from .utils import loggingHandler

__author__ = "Ishfaq Ahmed"
__email__ = "ishfaqahmed0837@gmail.com"
__description__ = ("Custom built utilities for general use",)
__all__ = (
    ProgressSaver,
    loggingHandler,
    processCSVInChunks,
    getFilePath,
    askYNQuestion,
    getInputFilePath,
    printArray,
    selectOptionQuestion,
    signature,
    asyncListDir,
    asyncReadJSON,
    createDirectory,
    readData,
    writeFile,
    progressBarIterator,
    concurrentFutures,
    initLogging,
)
